function ETglobal(start_year,end_year, ndvi_subdir, rn_subdir,...
    temp_subdir, ea_subdir,res)
%% Help
% ETglobal(start_year,end_year, ndvi_subdir, rn_subdir,...
%   temp_subdir, ea_subdir,res)
%
%Written by Grayson Badgley. Jet Propulsion Lab.
%Questions to: graysonbadgley@gmail.com
%Last Modified 24 Jan 2011
%
%This program calculates monthly, globally gridded evapotranspiration based
%on the method described in Fisher et al 2008 'Global estimates of the 
%land-atmosphere water flux based on monthly AVHRR and ISLSCP-II data,
%validated at 16 FLUXNET sites',Remote Sensing of Environment.
%
%Output units are in W-m^-2 in a geographic projection, with the upper-left corner
%of the first element representing 90 degrees North, 180 degrees West. 
%
%The program operates on a directory by directory basis. All input data
%should be organized in sub-directories, with proper units (see below) and 
%with uniform geographic gridding. To begin, change the MATLAB
%working directory to the parent directory of the input data. 
%
%Outputs monthly text files to current working directory.
%
%The specific function inputs are: 
%
%start_year: first year of data. Integer in form of yyyy.
%end_year: last year of  data. Integer in form of yyyy.
%ndvi_subdir: NDVI directory. input as string. Values from 0 to 1.
%rn_subdir: Net radiation directory. input as string. W m-2
%temp_subdir: Temperature directory. input as string. Celsius
%ea_subdir: Partial water vapor pressure directory. input as string. kPa.
%res: resolution of input data (must be uniform across datasets).
%Input real number to change resolution (e.g. for half deg, res = 0.5)
%
%Function requires installation of the findn function, which should be
%included in the zipped files you received. It can also be found at 
%http://www.mathworks.com/matlabcentral/fileexchange/3055-findn

%% Front Matter

%vector of all years analyzed
years = start_year:end_year; 

%Row/Col num based on input resolution. 
row			= 180*(1/res);
col			= 360*(1/res);

%Constants: See Fisher et al. 2008 p.902 for further explanation.

savi_mult   = 0.45; savi_add = 0.132;                        
savi_fun    = @(x) (x*savi_mult)+savi_add; %anon function to calc savi

h2o_base    = 0.61121; h2o_mult = 17.502; h2o_add = 240.97;  
h2osat_fun  = @(x) h2o_base*exp((h2o_mult*x)./(h2o_add+x)); %anon func

fApar_mult  = 1.2*1.136; fApar_add = 1.2*-0.04;
fApar_fun   = @(x) (fApar_mult*x)+fApar_add; %anon func

gamma_mult  = 240.97*17.502; gamma_add = 240.97;
gamma_fun   = @(x,y) (gamma_mult*x)./((y+gamma_add).^2);%anon func

ft_numer    = 1.14; ft_mult = 0.2; ft_less = 10;
fT_fun      = @(x,y) ft_numer./(1+exp((x-y-ft_less)*ft_mult));%anon func

fIpar_add   = -0.05;

kPar        = 0.3;

kRn         = 0.6;

delta		= 0.066;			%psychrometric constant

%Constraints: Want to filter the data of faulty values in the datasets
%(e.g. temp of 96 degC) placing min/max bounds on all inputs.

min_vi    = 0;
max_vi    = 1;

min_rn   = 0;
max_rn   = 280;

min_temp  = -90;
max_temp  = 55;

min_rh    = 0;
max_rh    = 1;

%Begin loop through each individual year processing (i-loop)
for i = 1:length(years)

    %% Load Files
	
	%Create list of all files will bring in for the year
	ndvi_files 	= dir([pwd,'\',ndvi_subdir,'\*',int2str(years(i)),'*']);
	rn_files 	= dir([pwd,'\',rn_subdir,'\*',int2str(years(i)),'*']);
	temp_files 	= dir([pwd,'\',temp_subdir,'\*',int2str(years(i)),'*']);
	ea_files 	= dir([pwd,'\',ea_subdir,'\*',int2str(years(i)),'*']);
	
    %calculate number of input months, necessary to accomodate incomplete
    %datasets
    num_mnth    = min([length(ndvi_files),length(rn_files),...
        length(temp_files),length(ea_files)]);
    
    %preallocate: place within loop to ensure scrub out all values from
    %previous year. size can also change from year to year based on months
    %available.
    
    ndvi        = nan(row,col,num_mnth);
    rn          = nan(row,col,num_mnth);
    temp        = nan(row,col,num_mnth);
    ea          = nan(row,col,num_mnth);
    esT         = nan(row,col,num_mnth);
    VPD         = nan(row,col,num_mnth);
    phen        = nan(row,col,num_mnth);
    savi        = nan(row,col,num_mnth);
    fApar       = nan(row,col,num_mnth);
    
    %Enter monthly loop that loads each file in order of list created with
    %dir() above.
	for m = 1:num_mnth
		
  		%Bring in the raw data for the month
 		ndvi(:,:,m)  = load([pwd,'\',ndvi_subdir,'\',ndvi_files(m).name]); 				
  		rn(:,:,m)	 = load([pwd,'\',rn_subdir,'\',rn_files(m).name]);		 				
 		temp(:,:,m)  = load([pwd,'\',temp_subdir,'\',temp_files(m).name]);			
  		ea(:,:,m)	 = load([pwd,'\',ea_subdir,'\',ea_files(m).name]); 				
 		
		%Calculate first order ancillary variables.                 
        savi(:,:,m)  = savi_fun(ndvi(:,:,m));
        fApar(:,:,m) = fApar_fun(savi(:,:,m));
        esT(:,:,m)   = h2osat_fun(temp(:,:,m));  %h2o saturation
		
		VPD(:,:,m)   = esT(:,:,m)-ea(:,:,m);	 %vapor pressure deficit
		
		%%Phenology variables: see Fisher et al. 2008 for help.
		phen_raw     = ((rn(:,:,m).*temp(:,:,m).*savi(:,:,m))./VPD(:,:,m));
		
		%filter out any negative numbers that might crop up.
		phen_raw(phen_raw<0)=0;
		phen(:,:,m)	= phen_raw;
		        
	end%months
	
	%% Yearly Maximum Calculations 
    %Model requires annual max phen and fApar. calculate those here. 
	fApar_max   = max(fApar,[],3);
	
	%getting max phen easy enough--but need to find where that occurs then
	%extract both the temp at that time and the month in which the max
	%occurs.
    
	%begin with pre-allocate of opt_T and opt_month.
	opt_T 		= nan(row,col);
	opt_mnth	= nan(row,col);
	
    %enter a loop that passes through months
	for k = 1:num_mnth
        
        %findn reports row and col wi layer k that equal to max of that
        %element along three dimensions. output is [i j]. type help findn.m
        %for seperate documentmation.
		indy    = findn(phen(:,:,k)==max(phen,[],3));
        
        %indy reports [i j] and since process one layer at a time know
        %that layer location defined by k. meaning that optT(i,j) should be
        %equal to temp(i,j,k). This loop simply moves through the rows of
        %indy one at a time to get [i j]. k comes from what step on inside
        %the 'outer' loop for k=1:num_mnth. 
        
        for jj = 1:length(indy)
            opt_T(indy(jj,1),indy(jj,2))   = temp(indy(jj,1),indy(jj,2),k);
            opt_mnth(indy(jj,1),indy(jj,2))= k;
        end %for jj = 1:length(indy)
        
	end%for k =1:num_mnth
	
	
	%% Monthly variables 
	for m = 1:size(temp,3) 
        
        %Fraction of PAR intercepted by total vegetation cover
		fIpar           = ndvi(:,:,m)+fIpar_add;
		
		%Leaf Area Index
		lai             = -log(1-fIpar)*(1/kPar);
		
		%partition the radiation between canopy and soil
		rns             = rn(:,:,m).*exp(-lai.*kRn);    %soil radiation
		rnc             = rn(:,:,m)-rns;                %canopy radiation
		
		%Percentage Green Canopy Fraction
		fG              = fApar(:,:,m)./fIpar;
        fG(fG<0)        = 0;     %bound btwn zero and one
        fG(fG>1)        = 1;
        fG(fIpar==0)    = 0; %filter out any values where fIpar = zero
				
		%%s and ssy
		s				= gamma_fun(esT(:,:,m),temp(:,:,m));
		ssy				= s./(s+delta);
		
		%Relative Humidity	
		RH				= ea(:,:,m)./esT(:,:,m);
        RH(RH>1)        = 1;
		
		%fSM--soil moisture constraint.
		fSM				= RH.^(VPD(:,:,m));
        fSM(fSM>1)      = 1;
        fSM(RH==0)      = 0;
		 
		%fwet--relative surface wetness
		fwet			= RH.^4;
		less_wet        = 1-fwet;
		
        %fT--plant temperature constraint
        fT				= fT_fun(opt_T,temp(:,:,m));
        fT(opt_T==0)    = 0;
		fT(fT>1)		= 1;
				
		%fM--plant moisture constraint 
		fM              = fApar(:,:,m)./fApar_max;
        
        %if current month less than the optimal monthfM_filter = 0. then
        %fill those zeros and others with one.
        fM(opt_mnth>m)  = 1;
             
        %if fApar never goes above 0 then fM also equal zero
        fM(fApar_max==0)= 0;
         
        %Prepare filters for invalid values from input data
        filter          = zeros(row,col);
        rn_filter       = zeros(row,col);
        
        vi_indy         = any(ndvi(:,:,m)>max_vi | ndvi(:,:,m)<min_vi,3);
        temp_indy       = any(temp(:,:,m)>max_temp | temp(:,:,m)<min_temp,3);
        RH_indy         = any(RH>max_rh | RH<min_rh,3);
        rn_indy         = any(rn(:,:,m)>max_rn | rn(:,:,m)<min_rn,3);
        
        filter(temp_indy)   = 1;
        filter(vi_indy)     = 1;
        filter(RH_indy)     = 1;
        rn_filter(rn_indy)  = 1;
        
        
		%% Calculate Monthly ET and Write Output
		LE_mnth = (fwet.*ssy.*rnc.*1.26)+fSM.*less_wet.*ssy.*rns.*1.26+...
            ssy.*rnc.*less_wet.*fT.*fG.*fM.*1.26+(fwet.*ssy.*rns)*1.26;

        %filter out bad data
        LE_mnth(filter==1)   = NaN;
        LE_mnth(rn_filter==1)= 0;
     
		%And then write the monthly ouput to pwd
    	dlmwrite([pwd,'\ucb_le_',int2str(years(i)),...
            '_',sprintf('%02d',m),'.txt'],LE_mnth,'delimiter', ' ',...
            'precision','%.3f');
    end %monthly vars

	
end %years

end %func